@@ -94,6 +94,13 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 #endif
 
+#ifdef __BEOS__
+# undef READ
+# undef WRITE
+# define READ(fd, buf, cnt) recv((fd), (buf), (cnt), 0)
+# define WRITE(fd, buf, cnt) send((fd), (buf), (cnt), 0)
+#endif
+
 /* mswindows.h defines these.  */
 #ifndef READ
 # define READ(fd, buf, cnt) read ((fd), (buf), (cnt))
@@ -178,4 +185,14 @@ void *memcpy ();
 #endif /* sun */
 #endif /* STDC_HEADERS */
 
+/* Some systems (Linux libc5, "NCR MP-RAS 3.0", and others) don't
+   provide MAP_FAILED, a symbolic constant for the value returned by
+   mmap() when it doesn't work.  Usually, this constant should be -1.
+   This only makes sense for files that use mmap() and include
+   sys/mman.h *before* sysdep.h, but doesn't hurt others.  */
+
+#ifndef MAP_FAILED
+# define MAP_FAILED ((void *) -1)
+#endif
+
 #endif /* SYSDEP_H */
